from mercurial.node import hex
from mercurial import merge
from mercurial import cmdutil

cmdtable = {}
command = cmdutil.command(cmdtable)


def normalizename(name):
    return name.replace('\\', '\\\\').replace('\n', '\\n').replace('\t', '\\t')


@command('sgwtstate')
def sgwtstate(ui, repo, **opts):
    ctx = repo[None]
    parents = [p for p in ctx.parents() if p.rev() >= 0]

    ui.write('heads\t', str(len(parents)), '\n')
    for p in parents:
        ui.write('head\t', hex(p.node()), '\n')

    mergestate = merge.mergestate.read(repo)
    mergestateactive = mergestate.active()

    if len(parents) >= 2:
        commonAncestor = repo.changelog.ancestor(parents[0].node(), parents[1].node())
        ui.write('mergeparent\t', hex(parents[1].node()), '\n')
        ui.write('commonancestor\t', hex(commonAncestor), '\n')
    elif len(parents) == 1 and mergestateactive and mergestate.otherctx is not None:
        commonAncestor = repo.changelog.ancestor(parents[0].node(), mergestate.otherctx.node())
        ui.write('mergeparent\t', hex(mergestate.otherctx.node()), '\n')
        ui.write('commonancestor\t', hex(commonAncestor), '\n')
    else:
        ui.write('mergeparent\t\n')
        ui.write('commonancestor\t\n')

    ui.write('headrev\t', str(parents[0].rev()) if len(parents) >= 1 else str(-1), '\n')

    tip = repo[len(repo) - 1] if len(repo) >= 1 else None
    ui.write('tip\t', hex(tip.node()) if tip is not None else '', '\n')
    ui.write('tiprev\t', str(tip.rev()) if tip is not None else '-1', '\n')

    branch = ctx.branch()
    ui.write('branch\t', normalizename(branch), '\n')

    bookmark = repo._activebookmark
    ui.write('bookmark\t', normalizename(bookmark) if bookmark is not None else '', '\n')

    ui.write('mergestate\t', str(mergestateactive), '\n')
    if mergestateactive:
        filetopaths = dict()
        for rtype, record in mergestate._readrecords():
            if rtype in 'FDC':
                r = record.split('\0')
                f, state, hash, lfile, afile, anode, ofile = r[0:7]
                filetopaths[f] = (lfile, ofile, afile)

        unresolved = list(mergestate.unresolved())
        ui.write('unresolved\t', str(len(unresolved)), '\n')
        for f in unresolved:
            paths = filetopaths[f]
            ui.write(f, '\t', (paths[0] + '\t' + paths[1] + '\t' + paths[2] if paths is not None else '\t\t'), '\n')

    ui.write('state\t')
    try:
        if repo.vfs.exists('graftstate'):
            repo.vfs.read('graftstate').splitlines()
            ui.write('grafting')
            return
    except IOError:
        pass

    try:
        if repo.vfs.exists('rebasestate'):
            wdparent = -1
            target = -1
            for i, l in enumerate(repo.vfs.read('rebasestate').splitlines()):
                if i == 0:
                    wdparent = repo[l].rev()
                elif i == 1:
                    target = repo[l].rev()

            ui.write('rebasing\n')
            ui.write('wdparent\t', hex(repo[wdparent].node()) if wdparent >= 0 else '', '\n')
            ui.write('wdparentrev\t', str(wdparent), '\n')
            ui.write('target\t', hex(repo[target].node()) if target >= 0 else '', '\n')
            ui.write('targetrev\t', str(target), '\n')
            return
    except IOError:
        pass

    if len(parents) >= 2:
        ui.write('merging\n')
    else:
        ui.write('normal\n')
